#ifndef __CONTEXT__
#define __CONTEXT__


namespace XMLIOSERVER
{
   class Context : public ObjectTemplate<Context>
   {
      public:

         Context(void) : ObjectTemplate<Context>(), ccalendar(NULL),
            fieldDef(NULL), fileDef(NULL), axisDef(NULL), gridDef(NULL)
         {/* Ne rien faire de plus */}

         Context(const string& _id) : ObjectTemplate<Context>(_id), ccalendar(NULL),
            fieldDef(NULL), fileDef(NULL), axisDef(NULL), gridDef(NULL)
         {/* Ne rien faire de plus */}

         static void ShowTree(ostream& os = std::clog)
         {
            os << NIndent << "<?xml version=\"1.0\"?>" << std::endl;
            os << NIndent << "<"<< Context::GetRootName() << ">" << std::endl;

            HashMap<string, StrHashMap<Context> > &AllListContext = Context::GetAllListObject();
            for (HashMap<string, StrHashMap<Context> >::Iterator it = AllListContext.begin(); it != AllListContext.end(); it++)
               // On sort chacun des contextes successivement.
            { Context::SetCurrentContext((*it).first); os << *((*it).second)[(*it).first] << std::endl; }

            os << NIndent << "</" << Context::GetRootName() << ">" << std::endl  ;
         }

         static void FreeMemory(void)
         {
            HashMap<string, StrHashMap<Context> > &AllListContext = Context::GetAllListObject();
            for (HashMap<string, StrHashMap<Context> >::Iterator it = AllListContext.begin(); it != AllListContext.end(); it++)
            { Context::SetCurrentContext((*it).first); delete ((*it).second)[(*it).first]; }
         }

         static void ResolveInheritance(void)
         {
            HashMap<string, StrHashMap<Context> > &AllListContext = Context::GetAllListObject();
            for (HashMap<string, StrHashMap<Context> >::Iterator it = AllListContext.begin(); it != AllListContext.end(); it++)
            {
               // Résolution des héritages descendants (càd des héritages de groupes) pour chacun des contextes.
               Context::SetCurrentContext((*it).first);
               ((*it).second)[(*it).first]->resolveDescInheritance();

               // Résolution des héritages par référence au niveau des fichiers.
               const std::vector<CFile*>& allFiles = CFile::GetCurrentListObject().getVector();
               for (unsigned int i = 0; i < allFiles.size(); i++) allFiles[i]->resolveFieldRefInheritance();
            }
         }

         static void SetCurrentContext(const string& id)
         {
            // On modifie le context courrant pour tout les ObjectTemplate
            Context::SetContext(id);

            // Changement de context pour les champs et groupes de champs.
            FieldGroup::SetContext(id);
            CField::SetContext(id);

            // Changement de context pour les fichiers et groupes de fichiers.
            FileGroup::SetContext(id);
            CFile::SetContext(id);

            // Changement de context pour les grilles et groupes de grilles.
            GridGroup::SetContext(id);
            CGrid::SetContext(id);

            // Changement de context pour les axes et groupes d'axes.
            AxisGroup::SetContext(id);
            CAxis::SetContext(id);
         }

         virtual void parse (XMLNode& _node)
         {
            THashAttributes attributes;

            /// PARSING POUR GESTION DES ENFANTS
            if (_node.getElementName().compare(Context::GetName()))
               WARNING("Le noeud est mal nommé mais sera traité comme un context !");

            if (!(_node.goToChildElement()))
               WARNING("Le context ne contient pas d'enfant !");
            else
            {
              do { // Parcours des contexts pour traitement.

                  string name = _node.getElementName();
                  attributes.clear();
                  _node.getAttributes(attributes);

                  if (attributes.end() != attributes.find("id"))
                  { WARNING("Le noeud de définition possède un identifiant, ce dernier ne sera pas pris en compte lors du traitement !"); }

                  if (name.compare(FieldDefinition::GetDefName()) == 0) // Parsing pour la définition des champs.
                  { fieldDef = CreateInstanceAndParse<FieldDefinition>(_node, FieldDefinition::GetDefName().c_str()); continue; }

                  if (name.compare(FileDefinition::GetDefName()) == 0) // Parsing pour la définition des fichiers.
                  { fileDef  = CreateInstanceAndParse<FileDefinition >(_node, FileDefinition ::GetDefName().c_str()); continue; }

                  if (name.compare(AxisDefinition::GetDefName()) == 0) // Parsing pour la définition des axes.
                  { axisDef  = CreateInstanceAndParse<AxisDefinition >(_node, AxisDefinition ::GetDefName().c_str()); continue; }

                  if (name.compare(GridDefinition::GetDefName()) == 0) // Parsing pour la définition des grilles.
                  { gridDef  = CreateInstanceAndParse<GridDefinition >(_node, GridDefinition ::GetDefName().c_str()); continue; }

                  WARNING("La définition est invalide, seules les champs, grilles, axes et fichiers peuvent être définis !");

               } while (_node.goToNextElement());

               _node.goToParentElement(); // Retour au parent
            }
         }

         static string GetName(void) {return ("context"); }
         static string GetRootName(void) {return ("simulation"); }

         virtual bool hasChild(void) const
         { return ((fieldDef != NULL) or (fileDef != NULL)  or (axisDef != NULL) or (gridDef != NULL)); }

         virtual void printChild(ostream& out) const
         {
            if(fieldDef != NULL) out << *(FieldGroup*)     fieldDef << std::endl;
            if(fileDef  != NULL) out << *(FileGroup*)      fileDef  << std::endl;
            if(axisDef  != NULL) out << *(AxisDefinition*) axisDef  << std::endl;
            if(gridDef  != NULL) out << *(GridDefinition*) gridDef  << std::endl;
         }

         virtual void resolveDescInheritance(const AttributRegistrar* _parent = 0)
         {
            if (_parent != 0) return;
            // Résolution des héritages descendants pour chacun des groupes de définitions.
            if(fieldDef != NULL) fieldDef->resolveDescInheritance();
            if(fileDef  != NULL) fileDef ->resolveDescInheritance();
            if(axisDef  != NULL) axisDef ->resolveDescInheritance();
            if(gridDef  != NULL) gridDef ->resolveDescInheritance();
         }

         FieldDefinition* getFieldDefinition(void) const { return (this->fieldDef); }
         FileDefinition * getFileDefinition(void)  const { return (this->fileDef ); }
         AxisDefinition * getAxisDefinition(void)  const { return (this->axisDef ); }
         GridDefinition * getGridDefinition(void)  const { return (this->gridDef ); }

         AbstractCalendar * getCalendar(void) const { return (this->ccalendar ); }
         AbstractCalendar * setCalendar(const string& _calName, const string& dateStr)
         {
            if (_calName.compare("D360")      == 0)
               return (ccalendar = new D360Calendar(dateStr));
            if (_calName.compare("AllLeap")   == 0)
               return (ccalendar = new AllLeapCalendar(dateStr));
            if (_calName.compare("NoLeap")    == 0)
               return (ccalendar = new NoLeapCalendar(dateStr));
            if (_calName.compare("Julian")    == 0)
               return (ccalendar = new JulianCalendar(dateStr));
            if (_calName.compare("Gregorian") == 0)
               return (ccalendar = new GregorianCalendar(dateStr));

            WARNING("L'identifiant "+_calName+" est inconnu, le calendrier grégorien sera choisi par défault pour le contexte "+getId());

            return (ccalendar = new GregorianCalendar(dateStr));
         }

         ~Context()
         {
            // Désallocation dynamique de mémoire pour chacun des groupes de définition si nécessaire.
            if(fieldDef != NULL) delete fieldDef;
            if(fileDef  != NULL) delete fileDef;
            if(axisDef  != NULL) delete axisDef;
            if(gridDef  != NULL) delete gridDef;
            // Désallocation dynamique de mémoire pour le calendrier associé au contexte courant si nécessaire.
            if(ccalendar !=  NULL) delete ccalendar;
         }

      private:

         AbstractCalendar* ccalendar;

         FieldDefinition*  fieldDef;
         FileDefinition*   fileDef;
         AxisDefinition*   axisDef;
         GridDefinition*   gridDef;

   }; //class Context
}// namespace XMLIOSERVER

#endif  // __CONTEXT__
